function Gear=GearDesign(Gear,plotflag)
 
% GearDesign returns the basic triangulated model of a 3D spur Gear. It
% requires as input a few standard parameter to insert in the Gear
% structure. Here you can find the explanation of all parameters, for those
% who are not confident with mechanics but are curious to use this utility
% I suggest:
% 
%         http://en.wikipedia.org/wiki/Gear
% 
% The INPUT paremters are:
% 
% The Gear structure
% 
% Gear.m=1;%modulus
% Gear.z=20;%number of teeth
% Gear.Alpha=20;%Pressure angle degree
% Gear.Addendum=1;%addendum/modulus ratio
% Gear.Dedendum=1.25;%dedendum/modulus ratio
% Gear.FaceWidth=10;%Face width
% Gear.ShaftDiameter=4;%shaft diameter [mm]
% Gear.numpoint=10;%number of points to approximate the tooth profile
% Gear.t=[];%here will be stored the triangulation
% Gear.p=[];%here will be stored points data
% 
% 
% plotflag: turn on a plot of the designed gear
% 
% 
% The OUTPUT parameter is the Gear structure but with filled Gear.t (triangles)
% and Gear.p (points).
% 
% Here is a an example
% 
% % BUILD THE GEAR STRUCTURE
% 
% Gear2.m=1;%modulus
% Gear2.z=30;%number of teeth
% Gear2.Alpha=20;%Pressure angle degree
% Gear2.Addendum=1;%addendum/modulus ratio
% Gear2.Dedendum=1.25;%dedendum/modulus ratio
% Gear2.FaceWidth=4;%face width
% Gear2.ShaftDiameter=8;%shaft diameter
% Gear2.numpoint=10;%points to approximate the tooth profile
% Gear2.t=[];%for the output
% Gear2.p=[];%for the output
% 
% % GEAR DESIGN
% Gear2=GearDesign(Gear2,true);
% 
% For problems, questions, suggestions,:
% 
%            giaccariluigi@msn.com
% 
% Created: 18/01/2009
% Last Update: 18/01/2009
% Author: Giaccari Luigi
% 




%error check
if nargin>2
    error('Too many input');
end

%Add here parameters check





if nargin<2
    plotflag=false;
end

Gear.Alpha=Gear.Alpha*2*pi/360;%%Pressure angle degree radiants
r=Gear.z*Gear.m/2;%primitive radius
rb=r*cos(Gear.Alpha);%base radius
%pitch=Gear.m*pi;%pitch

%rmin and max
rmin=r-Gear.Dedendum*Gear.m;
if Gear.ShaftDiameter>rmin
    error('Shaft diameter too large')
end

rmax=r+Gear.Addendum*Gear.m;


%getting the circle involve

if rb<rmin
    tetamin=acos(rb/rmin);
else
    tetamin=0;
end
tetamax=acos(rb/rmax);


teta=linspace(tetamin,tetamax,Gear.numpoint);

%polar coordinates of the circel involute
sigma=tan(teta)-teta;
ro=rb./cos(teta);

%cartesian coordinates
x=ro.*cos(sigma);
y=ro.*sin(sigma);



dpitch=pi/Gear.z/2;%diametral pitch





dteta=tan(Gear.Alpha)-Gear.Alpha;%angolo fra il punto dell'evolvente con ro=raggio primitivo
%e l'asse x, cos lo porto sull'asse x e poi lo ruoto di met passo
%angolare

%ruota il semiprofilo del dente per poi specchiarlo sull'assex
x=x*cos(dpitch+dteta)+y*sin(+dpitch+dteta);
y=-x*sin(dpitch+dteta)+y*cos(+dpitch+dteta);



%root circle

teta=linspace(-dpitch*2,-dpitch-dteta,ceil(Gear.numpoint/4));
tix=rmin*cos(teta);
tiy=rmin*sin(teta);






%outside circle
teta=linspace(sigma(end)-dpitch-dteta,0,ceil(Gear.numpoint/4));
tesx=rmax*cos(teta);
tesy=rmax*sin(teta);

%assemble all the piecies
x=[tix,x,tesx];
y=[tiy,y,tesy];



x=x';
y=y';

%tooth profile
%join two half profile
x=[x;flipud(x)];
y=[y;flipud(-y)];






%design each tooth
teeth=zeros(length(x)*Gear.z,3);
teta=linspace(2*pi-dpitch*4,0,Gear.z);
N=length(x);
i1=1;
i2=N;
for i=1:Gear.z

    teeth(i1:i2,1)=x*cos(teta(i))+y*sin(teta(i));
    teeth(i1:i2,2)=-x*sin(teta(i))+y*cos(teta(i));

    i1=i1+N;
    i2=i2+N;

end


if plotflag
    figure(101)
    title('Gear Profile','fontsize',14)
    hold on
    axis equal
    plot(teeth(:,1),teeth(:,2),'-k')

    circle([0,0],r,100,'r');
    circle([0,0],rb,100,'b');
    circle([0,0],rmin,100,'m');
    circle([0,0],rmax,100,'c');
    legend('Tooth profile','Pitch Circle','Base Circle',...
        'Root Circle','Outside Circle','Location', 'BestOutside');
end

%Getting the internal circle
N=size(teeth,1);
teta=linspace(0,2*pi,N);
Hole=zeros(N,3);
Hole(:,1)=Gear.ShaftDiameter/2*cos(teta);
Hole(:,2)=Gear.ShaftDiameter/2*sin(teta);

%Get the gear points
Gear.p=[teeth;Hole];
teeth(:,3)=Gear.FaceWidth;
Hole(:,3)=Gear.FaceWidth;
Gear.p=[Gear.p;teeth;Hole];

%Get the Gear triangles
Gear.t=[(1:N-1)',(2:N)',(N+1:2*N-1)';(2:N)',(N+1:2*N-1)',(N+2:2*N)'];%connetti la dentatura Gear.z=0
Gear.t=[Gear.t;Gear.t+2*N];%dentatura Gear.z>0;
Gear.t=[Gear.t;[(1:N-1)',(2:N)',(2*N+1:3*N-1)';(2:N)',(2*N+1:3*N-1)',(2*N+2:3*N)']];%le due dentature


if plotflag
    figure(102)
    hold on
    axis equal
    trisurf(Gear.t,Gear.p(:,1),Gear.p(:,2),Gear.p(:,3),'facecolor',[.5 .5 .5],'edgecolor','none')
    view(3)
end


end



function H=circle(center,radius,NOP,style)
%---------------------------------------------------------------------------------------------
% H=CIRCLE(CENTER,RADIUS,NOP,STYLE)
% This routine draws a circle with center defined as
% a vector CENTER, radius as a scaler RADIS. NOP is
% the number of points on the circle. As to STYLE,
% use it the same way as you use the rountine PLOT.
% Since the handle of the object is returned, you
% use routine SET to get the best result.
%
%   Usage Examples,
%
%   circle([1,3],3,1000,':');
%   circle([2,4],2,1000,'--');
%
%   Zhenhai Wang <zhenhai@ieee.org>
%   Version 1.00
%   December, 2002
%---------------------------------------------------------------------------------------------

if (nargin <3),
    error('Please see help for INPUT DATA.');
elseif (nargin==3)
    style='b-';
end;
THETA=linspace(0,2*pi,NOP);
RHO=ones(1,NOP)*radius;
[X,Y] = pol2cart(THETA,RHO);
X=X+center(1);
Y=Y+center(2);
H=plot(X,Y,style);
axis square;
end


